<?php
header('Access-Control-Allow-Origin: *');
header('Content-Type: application/json');
$method = $_SERVER['REQUEST_METHOD'];

if ($method === 'OPTIONS') {
    header('Access-Control-Allow-Methods: GET, POST, PUT, DELETE');
    header('Access-Control-Allow-Headers: Origin, Accept, Content-Type, X-Requested-With');
    exit();
}

include_once '../../config/Database.php';
include_once '../../models/Quote.php';

$database = new Database();
$db = $database->connect();

if (isset($_GET['author_id']) && isset($_GET['category_id'])) {
    $author_id = $_GET['author_id'];
    $category_id = $_GET['category_id'];
    echo getQuotesByAuthorAndCategory($db, $author_id, $category_id);
} elseif (isset($_GET['author_id'])) {
    $author_id = $_GET['author_id'];
    echo getQuotesByAuthorId($db, $author_id);
} elseif (isset($_GET['category_id'])) {
    $category_id = $_GET['category_id'];
    echo getQuotesByCategoryId($db, $category_id);
} elseif (isset($_GET['id'])) {
    $id = $_GET['id'];
    echo getQuoteById($db, $id);
} elseif (!isset($_GET['author_id']) && !isset($_GET['category_id']) && !isset($_GET['id'])){
    echo getAllQuotes($db);
} else {
    http_response_code(404);
    echo json_encode(array("message" => "Invalid request"));
}

function getAllQuotes($db) {
    $quote = new Quote($db);

    $stmt = $quote->read();
    $num = $stmt->rowCount();

    if($num > 0) {
        $quotes_arr=array();

        while ($row = $stmt->fetch(PDO::FETCH_ASSOC)) {
            $quote_item=array(
                "id" => $row['id'],
                "quote" => $row['quote'],
                "author" => isset($row['author_id']) ? $row['author_id'] : '',
                "category" => isset($row['category_id']) ? $row['category_id'] : ''
            );

            array_push($quotes_arr, $quote_item);
        }

        return json_encode($quotes_arr);
    } else {
        return json_encode(array("message" => 'No Quotes Found'));
    }
}

function getQuoteById($db, $id) {
    $quote = new Quote($db);
    $quote->id = $id;

    $stmt = $quote->readSingle();
    $num = $stmt->rowCount();

    if($num > 0) {
        $row = $stmt->fetch(PDO::FETCH_ASSOC);
        extract($row);
        $quote_item=array(
            "id" => $id,
            "quote" => $quote,
            "author" => $author_id,
            "category" => $category_id
        );

        return json_encode($quote_item);
    } else {
        return json_encode(array("message" => 'No Quotes Found'));
    }
}

// Parse author_id from query parameters
function getQuotesByAuthorId($db, $author_id) {
    $quote = new Quote($db);

    $stmt = $quote->getQuotesByAuthorId($author_id);
    $num = $stmt->rowCount();

    if ($num > 0) {
        $quotes_arr = array();

        while ($row = $stmt->fetch(PDO::FETCH_ASSOC)) {
            extract($row);
            $quote_item = array(
                "id" => $id,
                "quote" => $quote,
                "author" => $author_id,
                "category" => $category_id,
            );
        
            array_push($quotes_arr, $quote_item);
        }
        return json_encode($quotes_arr);
    } else {
        return json_encode(array("message" => 'No Quotes Found'));
    }
}

function getQuotesByCategoryId($db, $category_id) {
    $quote = new Quote($db);

    $stmt = $quote->getQuotesByCategoryId($category_id);
    $num = $stmt->rowCount();

    if ($num > 0) {
        $quotes_arr = array();

        while ($row = $stmt->fetch(PDO::FETCH_ASSOC)) {
            extract($row);
            $quote_item = array(
                "id" => $id,
                "quote" => $quote,
                "author" => $author_id,
                "category" => $category_id
            );
        
            array_push($quotes_arr, $quote_item);
        }
        return json_encode($quotes_arr);
    } else {
        return json_encode(array("message" => 'No Quotes Found'));
    }
}

function getQuotesByAuthorAndCategory($db, $author_id, $category_id) {
    $quote = new Quote($db);

    $stmt = $quote->readByAuthorAndCategory($author_id, $category_id);
    $num = $stmt->rowCount();

    if($num > 0) {
        $quotes_arr=array();

        while ($row = $stmt->fetch(PDO::FETCH_ASSOC)) {
            extract($row);
            $quote_item=array(
                "id" => $id,
                "quote" => $quote,
                "author" => $author_id,
                "category" => $category_id
            );
        
            array_push($quotes_arr, $quote_item);
        }

        return json_encode($quotes_arr);
    } else {
        return json_encode(array("message" => 'No Quotes Found'));
    }
}
?>